const logger = require('@pubsweet/logger')
const get = require('lodash/get')
const helpers = require('../../helpers/helpers')
const collectionHelper = require('../../helpers/Collection')
const teamHelper = require('../../helpers/Team')
const config = require('config')
const mailService = require('pubsweet-component-mail-service')
const userHelper = require('../../helpers/User')
const invitationHelper = require('../../helpers/Invitation')

const configRoles = config.get('roles')

module.exports = models => async (req, res) => {
  const { email, role } = req.body

  if (!helpers.checkForUndefinedParams(email, role)) {
    res.status(400).json({ error: 'Email and role are required' })
    logger.error('User ID and role are missing')
    return
  }

  if (!configRoles.collection.includes(role)) {
    res.status(400).json({ error: `Role ${role} is invalid` })
    logger.error(`invitation attempted on invalid role ${role}`)
    return
  }
  const reqUser = await models.User.find(req.user)
  if (email === reqUser.email && !reqUser.admin) {
    logger.error(`${reqUser.email} tried to invite his own email`)
    return res.status(400).json({ error: 'Cannot invite yourself' })
  }

  const collectionId = get(req, 'params.collectionId')
  let collection
  try {
    collection = await models.Collection.find(collectionId)
  } catch (e) {
    const notFoundError = await helpers.handleNotFoundError(e, 'collection')
    return res.status(notFoundError.status).json({
      error: notFoundError.message,
    })
  }
  const url = `${req.protocol}://${req.get('host')}`

  try {
    const user = await models.User.findByEmail(email)

    await teamHelper.setupManuscriptTeam(models, user, collectionId, role)

    // find if there already is a matching invitation in the collection
    const invitation = await collectionHelper.addInvitation(
      collection,
      user.id,
      role,
    )
    if (role === 'handlingEditor') {
      await collectionHelper.addHandlingEditor(collection, user, invitation)
    }
    const status = get(collection, 'status')
    if (role === 'reviewer' && status === 'he-assigned') {
      collection.status = 'reviewers-invited'
      await collection.save()
    }
    try {
      if (role === 'handlingEditor') {
        await mailService.setupAssignEmail(
          user.email,
          'assign-handling-editor',
          url,
        )
      } else if (role === 'reviewer') {
        await invitationHelper.setupReviewerInvitation(
          req,
          models,
          collection,
          user,
          mailService,
        )
      } else {
        return res.status(500).json({
          error: 'Something went wrong',
        })
      }

      return res.status(200).json(user)
    } catch (e) {
      logger.error(e)
      return res.status(500).json({ error: 'Email could not be sent.' })
    }
  } catch (e) {
    if (role === 'reviewer') {
      const newUser = await userHelper.setupNewUser(
        req.body,
        url,
        res,
        email,
        role,
        models.User,
        'invite',
      )
      if (newUser.error !== undefined) {
        return res.status(newUser.status).json({
          error: newUser.message,
        })
      }
      await teamHelper.setupManuscriptTeam(models, newUser, collectionId, role)
      await collectionHelper.addInvitation(collection, newUser.id, role)
      await invitationHelper.setupReviewerInvitation(
        req,
        models,
        collection,
        newUser,
        mailService,
      )
      return res.status(200).json(newUser)
    }
    const notFoundError = await helpers.handleNotFoundError(e, 'user')
    return res.status(notFoundError.status).json({
      error: notFoundError.message,
    })
  }
}
